package com.kexun.aop;

import com.alibaba.fastjson.JSONObject;
import com.kexun.annotation.HisModifyLog;
import com.kexun.entity.BusInfoModifyHis;
import com.kexun.enums.BusinessTypeEnum;
import com.kexun.service.BusInfoModifyHisService;
import com.kexun.utils.CompareObjectUtils;
import com.kexun.utils.SpringContextUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.*;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Configuration;
import org.springframework.util.ReflectionUtils;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

/**
 * Description:通过切面记录修改的相关历史记录
 *
 * @author lgs
 * @date 2023/05/16
 */
@Configuration
@Aspect
@Slf4j
public class HisModifyInfoAspect {

    @Autowired
    BusInfoModifyHisService busInfoModifyHisService;

    /**
     * 切点表达式,拦截所有注解HisModifyLog标识的方法
     */
    private final String DEFAULT_QUERY_POINT = "@annotation(com.kexun.annotation.HisModifyLog)";

    /**
     * 抽取公共的切点
     */
    @Pointcut(DEFAULT_QUERY_POINT)
    public void pointCut() {
    }
    /**
     * 环绕通知
     */
    @Around("pointCut()")
    public Object around(ProceedingJoinPoint joinPoint) {

        /**
         * 处理思路：
         * 1、通过joinPoint获取到当前controller的实体
         * 2、获取controller传递过来的修改的对象
         * 3、通过注解里设置，获取相应的serviceBean和相应的方法，查询数据库里的信息
         * 4、通过前端传递过来的参数和数据库里查询到的实体，进行一个对比，找到修改了哪些
         * 5、记录到bus_info_modify_his业务信息变更历史记录表中
         */
        // 拦截的实体类，就是当前正在执行的controller
        Object target = joinPoint.getTarget();
        // 拦截的方法名称。当前正在执行的方法
        String methodName = joinPoint.getSignature().getName();
        // 拦截的方法参数
        Object[] args = joinPoint.getArgs();
        //定义请求参数的类对象
        Object newObject = null;
        Map paramMap = new HashMap();
        //循环参数，根据类型进行判断
        for (Object arg : args) {
            //通过该方法可查询对应的object属于什么类型：String type = paramsObj.getClass().getName();
            if (arg instanceof String || arg instanceof JSONObject) {
                String str = (String) arg;
                //TODO
            } else if (arg instanceof Map) {
                //get请求，以map类型传参
                Map<String, Object> map = (Map<String, Object>) arg;
                //TODO
            } else if (arg instanceof Object) {
                try {
                    Map<String, Object> describe = PropertyUtils.describe(arg);
                    paramMap.putAll(describe);
                    newObject = arg;
                } catch (IllegalAccessException e) {
                    e.printStackTrace();
                } catch (InvocationTargetException e) {
                    e.printStackTrace();
                } catch (NoSuchMethodException e) {
                    e.printStackTrace();
                }
            }
        }
        Signature signature = joinPoint.getSignature();
        MethodSignature msig = null;
        if (!(signature instanceof MethodSignature)) {
            throw new IllegalArgumentException("该注解只能用于方法");
        }
        msig = (MethodSignature) signature;
        Class[] parameterTypes = msig.getMethod().getParameterTypes();
        // 获得被拦截的方法
        Method method = null;
        try {
            method = target.getClass().getMethod(methodName, parameterTypes);
        } catch (NoSuchMethodException e1) {
            log.error("ControllerLogAopAspect around error", e1);
        } catch (SecurityException e1) {
            log.error("ControllerLogAopAspect around error", e1);
        }
        if (null != method) {
            // 判断是否包含自定义的注解
            if (method.isAnnotationPresent(HisModifyLog.class) && newObject != null) {
                HisModifyLog hisModifyLog = method.getAnnotation(HisModifyLog.class);
                //获取注解相关属性
                //变更模块名称
                String module = hisModifyLog.module();
                //业务类型
                String businessType = hisModifyLog.businessType();
                //service的bean
                String serviceBean = hisModifyLog.serviceBean();
                //查询方法
                String queryMethod = hisModifyLog.queryMethod();
                //查询参数的参数类型
                String paramType = hisModifyLog.paramType();
                //参数key
                String paramName = hisModifyLog.paramKey();
                //是否为批量操作
                boolean isArray = hisModifyLog.paramIsArray();
                //从请求的参数中解析查询出key对应的value值
                Method mh = ReflectionUtils.findMethod(SpringContextUtil.getBean(serviceBean).getClass(), queryMethod, Long.class);
                //用spring bean获取操作前的参数,此处需要注意：传入的id类型与bean里面的参数类型需要保持一致
                Object id = paramMap.get(paramName);
                Object oldObject = ReflectionUtils.invokeMethod(mh, SpringContextUtil.getBean(serviceBean), id);
                //对新旧的对象进行对比

                String comparResult = CompareObjectUtils.compareTwoObjectForDesc(newObject, oldObject);
                //记录到bus_info_modify_his表中去
                BusInfoModifyHis busInfoModifyHis = new BusInfoModifyHis();
                //业务编号
                busInfoModifyHis.setBusinessNum(String.valueOf(id));
                //业务类型
                busInfoModifyHis.setBusinessType(businessType);
                //操作模块名称
                busInfoModifyHis.setModuleName(module);
                //操作简单描述
                busInfoModifyHis.setOperation(BusinessTypeEnum.findEnumByCode(businessType).name());
                //操作详细描述
                busInfoModifyHis.setDescription(comparResult);
                busInfoModifyHisService.save(busInfoModifyHis);
            }
        }
        Object object = null;
        //执行页面请求模块方法，并返回
        try {
            object = joinPoint.proceed();
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }
        return object;
    }
}